# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "PaymentTransactionStatusResponse",
    "_Links",
    "_LinksSelf",
    "AdditionalInformation",
    "Data",
    "DataCharges",
    "DataCustomer",
    "Details",
]


class _LinksSelf(BaseModel):
    href: Optional[str] = None
    """Hyperlink to access the financial payment transaction status."""


class _Links(BaseModel):
    self: Optional[_LinksSelf] = None


class AdditionalInformation(BaseModel):
    description: Optional[str] = None
    """Product Identifier."""

    name: Optional[str] = None
    """Product Name."""


class DataCharges(BaseModel):
    amount: Optional[float] = None

    payer: Optional[Literal["M", "C"]] = None
    """the bearer of the payment charges ie. [Merchant, Customer]"""


class DataCustomer(BaseModel):
    email: Optional[str] = None
    """Customer email."""

    firstname: Optional[str] = None
    """Customer First Name."""

    msisdn: Optional[str] = None
    """Customer Mobile number."""

    surname: Optional[str] = None
    """Customer surname."""


class Data(BaseModel):
    amount: Optional[float] = None
    """The financial transactional amount."""

    channel: Optional[str] = None
    """Channel of the payment request eg. USSD"""

    charges: Optional[DataCharges] = None
    """Total charges associated with a particular payment"""

    customer: Optional[DataCustomer] = None

    date: Optional[str] = None
    """payment date"""

    description: Optional[str] = None
    """can be a payer note, a merchant identifier ie. merchantId etc."""

    financial_transaction_id: Optional[str] = FieldInfo(alias="financialTransactionId", default=None)
    """The financial transaction ID. Generated by the back-end system."""

    fulfillment_status: Optional[str] = FieldInfo(alias="fulfillmentStatus", default=None)
    """The financial transaction fullfilment status."""

    product: Optional[str] = None
    """A product id, name etc."""

    request_type: Optional[str] = FieldInfo(alias="requestType", default=None)
    """Request Type."""

    status: Optional[str] = None
    """Payment Status."""

    transaction_description: Optional[str] = FieldInfo(alias="transactionDescription", default=None)
    """The financial transaction description."""

    transaction_ref_parent: Optional[str] = FieldInfo(alias="transactionRefParent", default=None)
    """The financial transaction reference parent."""


class Details(BaseModel):
    brand: Optional[str] = None
    """Telco Code."""

    fulfillment_msisdn: Optional[str] = FieldInfo(alias="fulfillmentMsisdn", default=None)
    """Fulfillment Msisdn."""

    issuer: Optional[str] = None
    """Source of funds scheme."""


class PaymentTransactionStatusResponse(BaseModel):
    api_links: Optional[_Links] = FieldInfo(alias="_links", default=None)
    """Relevant links to the financial payment transaction status."""

    additional_information: Optional[AdditionalInformation] = FieldInfo(alias="additionalInformation", default=None)

    correlator_id: Optional[str] = FieldInfo(alias="correlatorId", default=None)
    """
    Unique identifier in the client for the payment in case it is needed to
    correlate, a trace id associated with the caller
    """

    customer_id: Optional[str] = FieldInfo(alias="customerId", default=None)
    """Customer identifier, a terminal id etc."""

    data: Optional[Data] = None

    details: Optional[Details] = None

    provider_transaction_id: Optional[str] = FieldInfo(alias="providerTransactionId", default=None)
    """ID of the payment, generated by back-end system.

    This can be blank if the payment has previously been pre-approved.
    """

    sequence_no: Optional[str] = FieldInfo(alias="sequenceNo", default=None)
    """A unique id for tracing all requests"""

    status_code: Optional[str] = FieldInfo(alias="statusCode", default=None)
    """
    This is the MADAPI Canonical Error Code (it is 4 characters long and it is not
    the HTTP Status Code which is 3 characters long). Back-end system errors are
    mapped to specific canonical error codes which are returned. 0000 is for a
    success. More information on these mappings can be found on the MADAPI
    Confluence Page 'Response Codes'
    """

    status_message: Optional[str] = FieldInfo(alias="statusMessage", default=None)
    """This is a description of the status"""
